/*
 *  Copyright (C) 2007 Viktor Nordell <viktor.nordell@gmail.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Viktor Nordell <viktor.nordell@gmail.com>
 */
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include "gui_join_create_guide.h"
#include "gui_frame.h"
#include "gui_fx.h"
#include "io_ipc.h"
#include "i18n.h"
#include "main.h"

#define COLOR(x) (x*(65535/255))

typedef struct callback_data{
	MainPrivate *priv;
	GtkWidget *frame;
	GtkWidget *name;
	GtkWidget *pass;
}callback_data;

static GtkWidget* 	build_create_screen	(callback_data* data);
static GtkWidget* 	build_start_screen	(callback_data* data);
static GtkWidget*	build_join_screen	(callback_data* data);
static GtkWidget*	build_result_screen	(callback_data* data, gboolean success, gchar* text);
static GtkWidget*	build_button		(gchar* text, gchar* stock_id, GtkIconSize size);

static void			callback_create_screen	(GtkButton *button, callback_data* data);
static void			callback_back			(GtkButton *button, callback_data* data);
static void			callback_join_screen	(GtkButton *button, callback_data* data);
static void			callback_create			(GtkButton *button, callback_data* data);
static void			callback_join			(GtkButton *button, callback_data* data);

static GtkWidget*
build_button(gchar* text, gchar* stock_id, GtkIconSize size)
{
	GtkWidget *button;	
	GtkWidget *image;
	GtkWidget *label;
	GtkWidget *hbox;
	
	hbox = gtk_hbox_new(FALSE, 5);
	image = gtk_image_new_from_stock(stock_id, size);
	label = gtk_label_new(text);
	button = gtk_button_new();
	
	gtk_label_set_markup(GTK_LABEL(label), text);	
	
	gtk_container_add(GTK_CONTAINER(button), hbox);
	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), image, TRUE, FALSE, 0);	
	
	return button;
}

static GtkWidget*
build_result_screen(callback_data *data, gboolean success, gchar* text)
{
	GtkWidget *align;
	GtkWidget *hbox;
	GtkWidget *img;
	GtkWidget *vbox;
	GtkWidget *largelabel;
	GtkWidget *labelalign;
	GtkWidget *smalllabel;	
	
	align = gtk_alignment_new(0.5, 0.5, 1, 1);
	hbox = gtk_hbox_new(FALSE, 15);
	vbox = gtk_vbox_new(FALSE, 0);
	smalllabel = gtk_label_new(text);
	labelalign = gtk_alignment_new(0.5, 0.0, 1, 1);
	
	if(success){
		largelabel = gtk_label_new(_("<span font_desc='22' weight='ultrabold'>Success</span>"));
		img = gtk_image_new_from_stock(GTK_STOCK_DIALOG_INFO, GTK_ICON_SIZE_DIALOG);
	}else{
		largelabel = gtk_label_new(_("<span font_desc='22' weight='ultrabold'>Failure</span>"));
		img = gtk_image_new_from_stock(GTK_STOCK_DIALOG_ERROR, GTK_ICON_SIZE_DIALOG);
	}
	
	gtk_alignment_set_padding(GTK_ALIGNMENT(align), 30, 30, 40, 40);
	gtk_label_set_use_markup(GTK_LABEL(largelabel), TRUE);
	gtk_label_set_use_markup(GTK_LABEL(smalllabel), TRUE);
	
	gtk_container_add(GTK_CONTAINER(align), hbox);
	gtk_box_pack_start(GTK_BOX(hbox), img, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);
	
	gtk_box_pack_start(GTK_BOX(vbox), largelabel, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), labelalign, TRUE, TRUE, 0);
	
	gtk_container_add(GTK_CONTAINER(labelalign), smalllabel);
	
	gui_fx_set_and_create_style(largelabel);
	gui_fx_set_and_create_style(smalllabel);
	
	gtk_widget_show_all(align);
	
	return align;
}

static GtkWidget*
build_join_screen(callback_data *data)
{
	GtkWidget *hbox;
	GtkWidget *leftbox;
	GtkWidget *rightbox;
		
	GtkWidget *namealign;
	GtkWidget *nameframe;
	GtkWidget *namebox;
	GtkWidget *namelabel;
	GtkWidget *nameentry;
	
	GtkWidget *passwdalign;	
	GtkWidget *passwdframe;
	GtkWidget *passwdbox;
	GtkWidget *passwdlabel;
	GtkWidget *passwdentry;
	
	GtkWidget *join_button;
	GtkWidget *back_button;
	
	hbox = gtk_hbox_new(FALSE, 5);
	leftbox = gtk_vbox_new(FALSE, 4);
	rightbox = gtk_vbox_new(TRUE, 20);
	
	nameframe = gtk_frame_new(_("Network Name"));
	passwdframe = gtk_frame_new(_("Network Password"));
	
	namealign = gtk_alignment_new(0.5, 0.5, 1, 1);
	passwdalign = gtk_alignment_new(0.5, 0.5, 1, 1);
	
	gtk_alignment_set_padding(GTK_ALIGNMENT(namealign), 5, 5, 5, 5);
	gtk_alignment_set_padding(GTK_ALIGNMENT(passwdalign), 5, 5, 5, 5);
	
	namebox = gtk_vbox_new(TRUE, 0);
	passwdbox = gtk_vbox_new(TRUE, 0);
	
	namelabel = gtk_label_new(_("Please enter the name of the network you\nwant to join."));
	passwdlabel = gtk_label_new(_("Please enter the password for the network."));
	
	data->name = nameentry = gtk_entry_new();
	data->pass = passwdentry = gtk_entry_new();
	
	join_button = build_button(_("<b>Join</b>"), GTK_STOCK_GO_FORWARD, GTK_ICON_SIZE_BUTTON);
	back_button = build_button(_("<b>Back</b>"), GTK_STOCK_GO_BACK, GTK_ICON_SIZE_BUTTON);
	
	gtk_box_pack_start(GTK_BOX(hbox), leftbox, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), rightbox, TRUE, FALSE, 0);
	
	gtk_box_pack_start(GTK_BOX(leftbox), nameframe, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(leftbox), passwdframe, TRUE, TRUE, 0);
	
	gtk_container_add(GTK_CONTAINER(nameframe), namealign);
	gtk_container_add(GTK_CONTAINER(passwdframe), passwdalign);
	
	gtk_container_add(GTK_CONTAINER(namealign), namebox);
	gtk_container_add(GTK_CONTAINER(passwdalign), passwdbox);
	
	gtk_box_pack_start(GTK_BOX(namebox), namelabel, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(namebox), nameentry, FALSE, TRUE, 0);
	
	gtk_box_pack_start(GTK_BOX(passwdbox), passwdlabel, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(passwdbox), passwdentry, FALSE, TRUE, 0);
	
	gtk_box_pack_start(GTK_BOX(rightbox), join_button, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(rightbox), back_button, FALSE, FALSE, 0);

	gui_fx_set_and_create_style(nameentry);
	gui_fx_set_and_create_style(passwdentry);
	gui_fx_set_and_create_style(join_button);
	gui_fx_set_and_create_style(back_button);
	gui_fx_set_and_create_style(nameframe);
	gui_fx_set_and_create_style(passwdframe);
	gui_fx_set_and_create_style(gtk_frame_get_label_widget(GTK_FRAME(nameframe)));
	gui_fx_set_and_create_style(gtk_frame_get_label_widget(GTK_FRAME(passwdframe)));
	
	gtk_frame_set_shadow_type(GTK_FRAME(nameframe), GTK_SHADOW_ETCHED_IN);
	gtk_frame_set_shadow_type(GTK_FRAME(passwdframe), GTK_SHADOW_ETCHED_IN);

	g_signal_connect (back_button, "clicked",
			G_CALLBACK (callback_back), data);
	g_signal_connect (join_button, "clicked",
			G_CALLBACK (callback_join), data);
	
	gtk_widget_show_all(hbox);

	return 	hbox;
}

static GtkWidget*
build_create_screen(callback_data *data)
{
	GtkWidget *hbox;
	GtkWidget *leftbox;
	GtkWidget *rightbox;
		
	GtkWidget *namealign;
	GtkWidget *nameframe;
	GtkWidget *namebox;
	GtkWidget *namelabel;
	GtkWidget *nameentry;
	
	GtkWidget *passwdalign;	
	GtkWidget *passwdframe;
	GtkWidget *passwdbox;
	GtkWidget *passwdlabel;
	GtkWidget *passwdentry;
	
	GtkWidget *create_button;
	GtkWidget *back_button;
	
	hbox = gtk_hbox_new(FALSE, 5);
	leftbox = gtk_vbox_new(FALSE, 4);
	rightbox = gtk_vbox_new(TRUE, 20);
	
	nameframe = gtk_frame_new(_("Network Name"));
	passwdframe = gtk_frame_new(_("Network Password"));
	
	namealign = gtk_alignment_new(0.5, 0.5, 1, 1);
	passwdalign = gtk_alignment_new(0.5, 0.5, 1, 1);
	
	gtk_alignment_set_padding(GTK_ALIGNMENT(namealign), 5, 5, 5, 5);
	gtk_alignment_set_padding(GTK_ALIGNMENT(passwdalign), 5, 5, 5, 5);
	
	namebox = gtk_vbox_new(TRUE, 0);
	passwdbox = gtk_vbox_new(TRUE, 0);
	
	namelabel = gtk_label_new(_("Please select a name for the network."));
	passwdlabel = gtk_label_new(_(" The password is used to prevent\n random people from joining the network."));
	
	data->name = nameentry = gtk_entry_new();
	data->pass = passwdentry = gtk_entry_new();
	
	create_button = build_button(_("<b>Create</b>"), GTK_STOCK_GO_FORWARD, GTK_ICON_SIZE_BUTTON);
	back_button = build_button(_("<b>Back</b>"), GTK_STOCK_GO_BACK, GTK_ICON_SIZE_BUTTON);
	
	gtk_box_pack_start(GTK_BOX(hbox), leftbox, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), rightbox, TRUE, FALSE, 0);
	
	gtk_box_pack_start(GTK_BOX(leftbox), nameframe, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(leftbox), passwdframe, TRUE, TRUE, 0);
	
	gtk_container_add(GTK_CONTAINER(nameframe), namealign);
	gtk_container_add(GTK_CONTAINER(passwdframe), passwdalign);
	
	gtk_container_add(GTK_CONTAINER(namealign), namebox);
	gtk_container_add(GTK_CONTAINER(passwdalign), passwdbox);
	
	gtk_box_pack_start(GTK_BOX(namebox), namelabel, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(namebox), nameentry, FALSE, TRUE, 0);
	
	gtk_box_pack_start(GTK_BOX(passwdbox), passwdlabel, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(passwdbox), passwdentry, FALSE, TRUE, 0);
	
	gtk_box_pack_start(GTK_BOX(rightbox), create_button, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(rightbox), back_button, FALSE, FALSE, 0);

	gui_fx_set_and_create_style(nameentry);
	gui_fx_set_and_create_style(passwdentry);
	gui_fx_set_and_create_style(create_button);
	gui_fx_set_and_create_style(back_button);
	gui_fx_set_and_create_style(nameframe);
	gui_fx_set_and_create_style(passwdframe);
	gui_fx_set_and_create_style(gtk_frame_get_label_widget(GTK_FRAME(nameframe)));
	gui_fx_set_and_create_style(gtk_frame_get_label_widget(GTK_FRAME(passwdframe)));
	
	gtk_frame_set_shadow_type(GTK_FRAME(nameframe), GTK_SHADOW_ETCHED_IN);
	gtk_frame_set_shadow_type(GTK_FRAME(passwdframe), GTK_SHADOW_ETCHED_IN);

	g_signal_connect (back_button, "clicked",
			G_CALLBACK (callback_back), data);
	g_signal_connect (create_button, "clicked",
			G_CALLBACK (callback_create), data);			
	
	gtk_widget_show_all(hbox);

	return 	hbox;
}

static GtkWidget*
build_start_screen(callback_data *data)
{	
	GtkWidget *hbox;
	GtkWidget *vbox;
	
	GtkWidget *create_image;
	GtkWidget *join_box;
	GtkWidget *create_box;
	GtkWidget *join_button;	
	GtkWidget *create_button;
	
	hbox 	= gtk_hbox_new(FALSE, 0);
	vbox	= gtk_vbox_new(TRUE, 0);
	join_box = gtk_hbox_new(FALSE, 0);
	create_box = gtk_hbox_new(FALSE, 0);
	create_image = gtk_image_new_from_stock(GTK_STOCK_ADD, GTK_ICON_SIZE_DIALOG);
		
	create_button = build_button(_("<b>Create Network</b>"), GTK_STOCK_GO_FORWARD, GTK_ICON_SIZE_DND);
	join_button = build_button(_("<b>Join a Network</b>  "), GTK_STOCK_GO_FORWARD, GTK_ICON_SIZE_DND);
		
	gtk_box_pack_start(GTK_BOX(hbox), create_image, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, FALSE, 0);
	
	gtk_box_pack_start(GTK_BOX(vbox), join_box, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), create_box, FALSE, FALSE, 0);
	
	gtk_box_pack_end(GTK_BOX(join_box), gtk_label_new("           "), TRUE, TRUE, 0);
	gtk_box_pack_end(GTK_BOX(create_box), gtk_label_new("           "), TRUE, TRUE, 0);
	
	gtk_box_pack_start(GTK_BOX(join_box), join_button, TRUE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(create_box), create_button, TRUE, FALSE, 0);	
	
	gui_fx_set_and_create_style(join_button);
	gui_fx_set_and_create_style(create_button);
	
	g_signal_connect (create_button, "clicked",
			G_CALLBACK (callback_create_screen), data);
	g_signal_connect (join_button, "clicked",
			G_CALLBACK (callback_join_screen), data);
	
	gtk_widget_show_all(hbox);
	
	return hbox;
}

void
gui_join_create_guide_run(MainPrivate *priv)
{
	GtkWidget *window;
	GtkWidget *frame;
	GtkWidget *widget;
	callback_data *data;
	
	data	= g_malloc0(sizeof(MainPrivate));
	data->priv = priv;
	window 	= gtk_window_new(GTK_WINDOW_TOPLEVEL);
	frame   = data->frame = gui_frame_new();
	widget 	= build_start_screen(data);
	
	gui_frame_set_default_handlers(GUI_FRAME(frame));
	gtk_container_add(GTK_CONTAINER(window), frame);
	gtk_container_add(GTK_CONTAINER(frame), widget);
	gtk_window_set_decorated(GTK_WINDOW(window), FALSE);
	gtk_widget_set_size_request(window, 450, 200);
	gtk_widget_show_all(window);
}

static void
callback_back(GtkButton *button, callback_data *data)
{
	GtkWidget *old;
	GtkWidget *new;
	
	new = build_start_screen(data);
	old = gtk_bin_get_child(GTK_BIN(data->frame));	
	
	gtk_container_remove(GTK_CONTAINER(data->frame), old);	
	gtk_container_add(GTK_CONTAINER(data->frame), new);	
}

static void
callback_join_screen(GtkButton *button, callback_data *data)
{
	GtkWidget *old;
	GtkWidget *new;
	
	new = build_join_screen(data);
	old = gtk_bin_get_child(GTK_BIN(data->frame));
	
	gtk_container_remove(GTK_CONTAINER(data->frame), old);	
	gtk_container_add(GTK_CONTAINER(data->frame), new);	
}

static void
callback_join(GtkButton *button, callback_data *data)
{
	GtkWidget *old;
	GtkWidget *new;
	gchar* name;
	gchar* pass;
	
	name = g_strdup(gtk_entry_get_text(GTK_ENTRY(data->name)));
	pass = g_strdup(gtk_entry_get_text(GTK_ENTRY(data->pass)));
	
	ipc_network_join(data->priv->connection, name, pass);
	
	switch(data->priv->connection->error){
		case IPC_ERROR_NON_EXISTENT_NETWORK:
			new = build_result_screen(data, FALSE, _("<span font_desc='10' style='italic' weight='light'>Network does not exist</span>"));
			break;
		case IPC_ERROR_WRONG_PASSWD:
			new = build_result_screen(data, FALSE, _("<span font_desc='10' style='italic' weight='light'>Wrong password</span>"));
			break;
		case IPC_ERROR_NO_ERROR:
			new = build_result_screen(data, TRUE, _("<span font_desc='10' style='italic' weight='light'>Network successfully joined!</span>"));
			break;
        case IPC_ERROR_NETWORK_FULL:
            new = build_result_screen(data, FALSE, _("<span font_desc='10' style='italic' weight='light'>Network is full</span>"));
            break;
		default:
			new = build_result_screen(data, FALSE, _("<span font_desc='10' style='italic' weight='light'>Unknown error, sorry...</span>"));
			break;		
	}	
	old = gtk_bin_get_child(GTK_BIN(data->frame));
	gtk_container_remove(GTK_CONTAINER(data->frame), old);	
	gtk_container_add(GTK_CONTAINER(data->frame), new);	
}

static void
callback_create(GtkButton *button, callback_data *data)
{
	GtkWidget *old;
	GtkWidget *new;
	gchar* name;
	gchar* pass;
	
	name = g_strdup(gtk_entry_get_text(GTK_ENTRY(data->name)));
	pass = g_strdup(gtk_entry_get_text(GTK_ENTRY(data->pass)));
	
	ipc_network_create(data->priv->connection, name, pass);
	
	switch(data->priv->connection->error){
		case IPC_ERROR_NETWORK_EXIST:
			new = build_result_screen(data, FALSE, _("<span font_desc='10' style='italic' weight='light'>Network already exists</span>"));
			break;
		case IPC_ERROR_NO_ERROR:
			new = build_result_screen(data, TRUE, _("<span font_desc='10' style='italic' weight='light'>Network successfully created!</span>"));
			break;
		default:
			new = build_result_screen(data, FALSE, _("<span font_desc='10' style='italic' weight='light'>Unknown error, sorry...</span>"));
			break;		
	}	
	old = gtk_bin_get_child(GTK_BIN(data->frame));
	gtk_container_remove(GTK_CONTAINER(data->frame), old);	
	gtk_container_add(GTK_CONTAINER(data->frame), new);	
}

static void
callback_create_screen(GtkButton *button, callback_data *data)
{
	GtkWidget *old;
	GtkWidget *new;
	
	new = build_create_screen(data);
	old = gtk_bin_get_child(GTK_BIN(data->frame));
	
	gtk_container_remove(GTK_CONTAINER(data->frame), old);	
	gtk_container_add(GTK_CONTAINER(data->frame), new);	
}
