/*
 *  Copyright (C) 2008 Viktor Nordell <viktor.nordell@gmail.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Viktor Nordell <viktor.nordell@gmail.com>
 */
#include <stdio.h>
#include <stdlib.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include "gui_settings_window_callbacks.h"
#include "gui_top_area.h"
#include "io_settings.h"
#include "io_ipc.h"
#include "main.h"

void
gui_settings_window_font_callback(GtkFontButton *widget, gpointer user_data)
{
    IOSettings *settings = IO_SETTINGS(user_data);
    
    settings->fontstr = g_strdup(gtk_font_button_get_font_name(widget));
    
    io_settings_updated(settings);
}

void
gui_settings_window_color_callback(GtkColorButton *widget, gpointer user_data)
{
    IOSettings *settings = IO_SETTINGS(user_data);
    
    gtk_color_button_get_color(widget, &settings->main_window_color);    

    io_settings_updated(settings);
}

void 
gui_settings_window_quit_dialog_callback(GtkToggleButton *togglebutton, gpointer user_data)
{
	IOSettings *settings;	
	settings = IO_SETTINGS(user_data);
	
	settings->show_quit_dialog = gtk_toggle_button_get_active(togglebutton);
	io_settings_updated(settings);
}

gboolean
gui_settings_window_nick_callback(GtkWidget *widget, GdkEventFocus *event, MainPrivate* priv)
{
    const gchar* nick;

    nick = gtk_entry_get_text(GTK_ENTRY(widget));
    
    ipc_set_nick(priv->connection, nick);
    gui_top_area_set_nick(GUI_TOP_AREA(priv->top_area), nick);
    
    return FALSE;
}

void 
gui_settings_window_effect_callback(GtkToggleButton *togglebutton, gpointer user_data)
{
	IOSettings *settings;	
	settings = IO_SETTINGS(user_data);
	
	settings->show_opacity = gtk_toggle_button_get_active(togglebutton);
	io_settings_updated(settings);
}

void 
gui_settings_window_opacity_callback(GtkSpinButton *spinbutton, gpointer user_data)
{
	IOSettings *settings;	
	settings = IO_SETTINGS(user_data);
	
	settings->opacity = gtk_spin_button_get_value(spinbutton);
	io_settings_updated(settings);
}

void 
gui_settings_window_show_taskbar_callback(GtkToggleButton *togglebutton, gpointer user_data)
{
	IOSettings *settings;	
	settings = IO_SETTINGS(user_data);
	
	settings->show_taskbar = gtk_toggle_button_get_active(togglebutton);
	io_settings_updated(settings);
}

void 
gui_settings_window_show_offline_callback(GtkToggleButton *togglebutton, gpointer user_data)
{
	IOSettings *settings;	
	settings = IO_SETTINGS(user_data);
	
	settings->show_offline = gtk_toggle_button_get_active(togglebutton);
	io_settings_updated(settings);
}

gboolean
gui_settings_window_vnc_entry_callback(GtkWidget *widget, GdkEventFocus *event, gpointer user_data)
{
    IOSettings *settings = IO_SETTINGS(user_data);
        	
	if(settings->cmd_vnc != NULL) g_free(settings->cmd_vnc);
	settings->cmd_vnc = g_strdup(gtk_entry_get_text(GTK_ENTRY(widget)));
	io_settings_updated(settings);
		
	return FALSE;
}

gboolean
gui_settings_window_sudo_entry_callback(GtkWidget *widget, GdkEventFocus *event, gpointer user_data)
{
    IOSettings *settings = IO_SETTINGS(user_data);
       
	if(settings->cmd_sudo != NULL) g_free(settings->cmd_sudo);
	settings->cmd_sudo = g_strdup(gtk_entry_get_text(GTK_ENTRY(widget)));
	io_settings_updated(settings);
		
	return FALSE;
}

gboolean
gui_settings_window_tuncfg_entry_callback(GtkWidget *widget, GdkEventFocus *event, gpointer user_data)
{
    IOSettings *settings = IO_SETTINGS(user_data);    
    
    if(settings->cmd_tuncfg != NULL) g_free(settings->cmd_tuncfg);
	settings->cmd_tuncfg = g_strdup(gtk_entry_get_text(GTK_ENTRY(widget)));
	io_settings_updated(settings);	
	
	return FALSE;
}

gboolean
gui_settings_window_hamachi_entry_callback(GtkWidget *widget, GdkEventFocus *event, gpointer user_data)
{
    IOSettings *settings = IO_SETTINGS(user_data);
        
    if(settings->cmd_hamachi != NULL) g_free(settings->cmd_hamachi);
	settings->cmd_hamachi = g_strdup(gtk_entry_get_text(GTK_ENTRY(widget)));
	io_settings_updated(settings);
		
	return FALSE;
}

gboolean    
gui_settings_window_browser_entry_callback(GtkWidget *widget, GdkEventFocus *event, gpointer user_data)
{
    IOSettings *settings = IO_SETTINGS(user_data);
	
	if(settings->cmd_browser != NULL) g_free(settings->cmd_browser);
	settings->cmd_browser = g_strdup(gtk_entry_get_text(GTK_ENTRY(widget)));	
	io_settings_updated(settings);
		
	return FALSE;
}

gboolean
gui_settings_window_terminal_entry_callback(GtkWidget *widget, GdkEventFocus *event, gpointer user_data)
{
    IOSettings *settings = IO_SETTINGS(user_data);
    
	if(settings->cmd_terminal != NULL) g_free(settings->cmd_terminal);
	settings->cmd_terminal = g_strdup(gtk_entry_get_text(GTK_ENTRY(widget)));	
	io_settings_updated(settings);
		
	return FALSE;
}

gboolean    
gui_settings_window_hamachi_folder_entry_callback(GtkWidget *widget, GdkEventFocus *event, gpointer user_data)
{
    IOSettings *settings = IO_SETTINGS(user_data);
    
	if(settings->hamachi_folder != NULL) g_free(settings->hamachi_folder);
	settings->hamachi_folder = g_strdup(gtk_entry_get_text(GTK_ENTRY(widget)));	
	io_settings_updated(settings);
	
	return FALSE;
}

gboolean    
gui_settings_window_keepalive_entry_callback(GtkWidget *widget, GdkEventFocus *event, MainPrivate* priv)
{
    int value;
    
    value = atoi(gtk_entry_get_text(GTK_ENTRY(widget)));
    
    if((value < 0) || (value > 10000)){
        gtk_entry_set_text(GTK_ENTRY(widget), "");
    }else{
        priv->hsettings->keep_alive = value;
    }
    
	return FALSE;
}

gboolean    
gui_settings_window_udpport_entry_callback(GtkWidget *widget, GdkEventFocus *event, MainPrivate* priv)
{
    int value;
    
    value = atoi(gtk_entry_get_text(GTK_ENTRY(widget)));
    
    if((value < 1024) || (value > 65536)){
        gtk_entry_set_text(GTK_ENTRY(widget), "");
    }else{
        priv->hsettings->udp_port = value;
    }
    
	return FALSE;
}
