/*
 *  Copyright (C) 2007 Viktor Nordell <viktor.nordell@gmail.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Viktor Nordell <viktor.nordell@gmail.com>
 */
#ifndef _IO_SETTINGS_H
#define _IO_SETTINGS_H

#include <glib.h>
#include <gdk/gdk.h>
#include <glib-object.h>

G_BEGIN_DECLS

#define HAMACHI_SETTINGS_FILE "/.hamachi-gui.conf"

#define IO_TYPE_SETTINGS			(io_settings_get_type ())
#define IO_SETTINGS(obj)			(G_TYPE_CHECK_INSTANCE_CAST ((obj), IO_TYPE_SETTINGS, IOSettings))
#define IO_SETTINGS_CLASS(obj)		(G_TYPE_CHECK_CLASS_CAST 	((obj), IO_TYPE_SETTINGS, IOSettingsClass))
#define IO_IS_SETTINGS(obj)			(G_TYPE_CHECK_INSTANCE_TYPE ((obj), IO_TYPE_SETTINGS))
#define IO_IS_SETTINGS_CLASS(obj)	(G_TYPE_CHECK_CLASS_TYPE 	((obj), IO_TYPE_SETTINGS))
#define IO_SETTINGS_GET_CLASS		(G_TYPE_INSTANCE_GET_CLASS 	((obj), IO_TYPE_SETTINGS, IOSettingsClass))

typedef struct _IOSettings			IOSettings;
typedef struct _IOSettingsClass		IOSettingsClass;

struct _IOSettings
{
	GObject parent;
	
	gboolean show_ip;
	gboolean show_nick;  
	gboolean show_taskbar;  
	gboolean show_offline;  	
	gboolean show_opacity;
	gboolean show_quit_dialog;
	gdouble opacity;  	
	gint update_time;
	
	GdkColor main_window_color;
	gchar* fontstr;
	
	gchar* cmd_vnc;		
	gchar* cmd_sudo;	
	gchar* cmd_tuncfg;  
	gchar* cmd_hamachi; 
	gchar* cmd_browser; 
	gchar* cmd_terminal;
        
    gchar* hamachi_folder;
};

struct _IOSettingsClass
{
	GObjectClass parent_class;
};

IOSettings*	io_settings_new 			(void);
IOSettings*	io_settings_new_with_path	(gchar* path);
void		io_settings_updated			(IOSettings *settings);
void		io_settings_write_settings	();
GType io_settings_get_type (void) G_GNUC_CONST;

G_END_DECLS

#endif
